import { DeclareConstructor } from "dojo/_base/declare";

interface _FormWidgetMixin {
    /**
     * Name used when submitting form; same as "name" attribute or plain HTML elements
     */
    name: string;

    /**
     * Corresponds to the native HTML `<input>` element's attribute.
     */
    alt: string;

    /**
     * Corresponds to the native HTML `<input>` element's attribute.
     */
    value: any;

    /**
     * Corresponds to the native HTML `<input>` element's attribute.
     */
    type: string;

    /**
     * Apply aria-label in markup to the widget's focusNode
     */
    'aria-label': string;

    /**
     * Order fields are traversed when user hits the tab key
     */
    tabIndex: number;

    /**
     * Should this widget respond to user input?
     * In markup, this is specified as "disabled='disabled'", or just "disabled".
     */
    disabled: boolean;

    /**
     * Fires onChange for each value change or only on demand
     */
    intermediateChanges: boolean;

    /**
     * On focus, should this widget scroll into view?
     */
    scrollOnFocus: boolean;

    /**
     * Tells if this widget is focusable or not.  Used internally by dijit.
     */
    isFocusable(): boolean;

    /**
     * Put focus on this widget
     */
    focus(): void;

    /**
     * Compare 2 values (as returned by get('value') for this widget).
     */
    compare(val1: any, val2: any): number;

    /**
     * Callback when this widget's value is changed.
     */
    onChange(value: string): void;

    /**
     * Overrides _Widget.create()
     */
    create(params?: any, srcNodeRef?: HTMLElement): void;

    destroy(preserveDom?: boolean): void;

    set(name: 'disabled', value: boolean): this;
    set(name: string, value: any): this;
    set(values: Object): this;
}

type _FormWidgetMixinConstructor = DeclareConstructor<_FormWidgetMixin>;
declare const _FormWidgetMixin: _FormWidgetMixinConstructor;
export = _FormWidgetMixin;
